package Pseudorandom_co_uk::Pseudosite::CopyConstructor;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

use strict;
use warnings;
use subs qw();
use vars qw(@ISA);
use utf8;

require Pseudorandom_co_uk::Pseudosite::Constructor;
@ISA = qw(Pseudorandom_co_uk::Pseudosite::Constructor);

sub construct
{
	my $self = shift;
	my $node = shift;

	my $in_name = $node->attr('infile');
	my $out_name = $node->attr('outfile');

	my $other = $node->attr('parent');

	if ( !( -f $in_name) )
	{
		die "Input $in_name to copier doesn't seem to exist, aborting";
	}

	if ( (! -f $out_name) or $self->lastmodified($in_name) > $self->lastmodified($out_name) )
	{
		print "copy: $out_name\n" if $self->verbosity > 0;

		my $dir = $out_name;
		my @list;
		while( $dir =~ m!/[^/]+$! )
		{
			$dir =~ s!^(.*)/[^/]+$!$1!;
			unshift @list, $dir;
		}
		foreach $dir (@list)
		{
			unless (-d $dir or $dir eq '')
			{
				print "$dir doesn't exist, creating it...\n" if $self->verbosity > 0;
				mkdir $dir or die "Couldn't create $dir";
			}
		}
		local $\;
		local *MYFILE;
		local *OUTFILE;
		$\ = '';
		if(open MYFILE, '<', $in_name)
		{
			binmode MYFILE;
			if(open OUTFILE, '>', $out_name)
			{
				binmode OUTFILE;
				my $count;
				my $buffer;
				while(read MYFILE, $buffer, 1024) # read 1KB chunks
				{
					print OUTFILE $buffer;
				}
				close OUTFILE;
				close MYFILE;
			} else {
				close MYFILE;
				die "Couldn't open $out_name for writing";
			}
		} else {
			die "Couldn't open $in_name for reading";
		}
	}
}
1;
