package Pseudorandom_co_uk::Pseudosite::PseudoTreeNode;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

sub xml
{
	my $tag = shift;
	my $retval = '';
	my $child;
	if (ref($tag))
	{
		$retval .= '<';
		$retval .= Pseudorandom_co_uk::Pseudosite::PseudoTreeText::xml_escape($tag->{type});
		my ($attr,$val,$a);
		foreach $attr (keys %{$tag->{attr}})
		{
			$attr = '' unless defined $attr;
			$tag->{attr}->{$attr} = '' unless defined $tag->{attr}->{$attr};
			$val = Pseudorandom_co_uk::Pseudosite::PseudoTreeText::xml_escape($tag->{attr}->{$attr});
			$a = Pseudorandom_co_uk::Pseudosite::PseudoTreeText::xml_escape($attr);
			$retval .= " $a=\"$val\"";
		}
		$retval .= ">";
		if(ref($tag->{kids}) eq 'ARRAY')
		{
			$retval .= "\n";
			my @kids = @{$tag->{kids}};
			while(@kids)
			{
				$child = shift @kids;
				$retval .= $child->xml;
			}
		}
		$retval .= '</'.Pseudorandom_co_uk::Pseudosite::PseudoTreeText::xml_escape($tag->{type}).'>';
		$retval .= "\n";

		return $retval;
	}
}

sub text
{
# Returns all the plain text contained in this element, unescaped
	my $tag = shift;
	my $retval = '';
	if (ref($tag))
	{
		if(ref($tag->{kids}) eq 'ARRAY')
		{
			my @kids = @{$tag->{kids}};
			my $child;
			while(@kids)
			{
				$child = shift @kids;
				$retval .= $child->text;
			}
		}
		return $retval;
	}
}

sub new
{
	my $class = shift;
	die "no class!" unless $class;
	my $tag = shift;
	my $ns = shift || '';
	my $parent = shift;
	my $element = { type => $tag, ns => $ns,
		kids => [], attr => {}, parent => $parent, };
	push @{$element->{parent}->{kids}}, $element;
	my $attr = shift;
	$element->{attr} = $attr if ref($attr) eq 'HASH';
	return bless $element, $class;
}

sub attr
{
	my $self = shift;
	my $attr = shift;
	my $val = shift;
	return undef unless ref($self);
	$self->{attr}->{$attr} = $val if defined $val;
	return $self->{attr}->{$attr};
}

sub attr_list
{
	my $self = shift;
	return undef unless ref($self);
	return keys %{$self->{attr}};
}

sub kids
{
	my $self = shift;
	return undef unless ref($self);
	return $self->{kids};
}

sub parent
{
	my $self = shift;
	return undef unless ref($self);
	return $self->{parent};
}

sub force_change_parent
{
	my $self = shift;
	my $new_parent = shift;
	$self->{parent} = $new_parent;
	return $self->{parent};
}

sub type
{
	my $self = shift;
	return undef unless ref($self);
	return $self->{type};
}

sub fulltype
{
	my $self = shift;
	return undef unless ref($self);
	return "{$self->{ns}}$self->{type}";
}

sub ns
{
	my $self = shift;
	return undef unless ref($self);
	return $self->{ns};
}
1;
