package Pseudorandom_co_uk::Pseudosite::PseudoTreeParser;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

use vars qw(@_ $_);
use strict;
use warnings;
use utf8;
use Pseudorandom_co_uk::Pseudosite::PseudoTreeNode;
use Pseudorandom_co_uk::Pseudosite::PseudoTreeText;
use XML::Parser;

# A handler for XML::Parser

# Based on XML::Parser's built-in Tree handler by Larry Wall <larry@wall.org>
# and Clark Cooper <coopercc@netheaven.com>

# Format: element == hash reference, cdata == scalar string.
# Structure is as follows:
# <tag attrib="value" attr2="foo"/> goes to
# $element = { type => "tag",
#             kids => [],
#             attr => { "attrib" => "value", "att2" => "foo", },
#             parent => ($parent_tag_or_undef),
#           };
# <b><i>bold italic</i> <tt>etc</tt></b> goes to:
# $element = { type => "b",
#             kids => [
#                        { type => "i", kids => [ "bold italic" ], attr => {}, parent => $element, },
#                        " ",
#                        { type => "tt", kids => [ "etc" ], attr => {}, parent => $element, },
#             ],
#             attr => {},
#           };

# FIXME: uses $expat->{PseudoTree} for storage (following LW/CC's precedent)
sub Parsefile {
	my $parser = new XML::Parser(Style => 'Pseudorandom_co_uk::Pseudosite::PseudoTreeParser', Namespaces=>1,);
#	my $f = shift;
#	my $r = $parser->parsefile($f);
#	print "---------- $f ----------\n";
#	print $r->xml('namespaces');
#	<STDIN>;
#	return $r;
	my $r = $parser->parsefile(shift);
}

sub Init {
  my $expat = shift;
  $expat->{PseudoTree} = {
    Root => undef, Current => undef,
  };
}

sub Start {
  my $expat = shift;
  my $tag = shift;
  my $att;
  my $val;
  my $element = new Pseudorandom_co_uk::Pseudosite::PseudoTreeNode $tag,$expat->namespace($tag),$expat->{PseudoTree}->{Current};
  $expat->{PseudoTree}->{Root} = $element unless (defined($expat->{PseudoTree}->{Root}));
  $expat->{PseudoTree}->{Current} = $element;
  $att = shift;
  $val = shift;
  while(defined $att and defined $val)
  {
  	$element->{attr}->{$att} = $val;
  	$att = shift;
  	$val = shift;
  }
}

sub End {
  my $expat = shift;
  my $tag = shift;
  die "Malformed XML" unless $tag eq $expat->{PseudoTree}->{Current}->type();
  $expat->{PseudoTree}->{Current} = $expat->{PseudoTree}->{Current}->parent();
}

sub Char {
  my $expat = shift;
  my $text = shift;
  my $clist = $expat->{PseudoTree}->{Current}->kids();
  my $pos = $#$clist;

  if ($pos > 1 and $clist->[$pos]->isa("Pseudorandom_co_uk::Pseudosite::PseudoTreeText") ) {
    $clist->[$pos]->append($text);
  } else {
    push @$clist, new Pseudorandom_co_uk::Pseudosite::PseudoTreeText $text;
  }
}

sub Final {
  my $expat = shift;
  delete $expat->{PseudoTree}->{Current};
  my $result = $expat->{PseudoTree}->{Root};
  delete $expat->{PseudoTree}->{Root};
  delete $expat->{PseudoTree};
  return $result;
}

1;
