package Pseudorandom_co_uk::Pseudosite::PseudoTreeText;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

use vars qw(@_ $_);
use strict;
use warnings;
use utf8;

sub is_xml_safe
{
	my $c = shift;
	my $o = ord($c);
	return 1 if ($c eq "\n" or $c eq "\r" or $c eq "\t" or $c eq "\f");
	return 0 if ($o < 31); # ctlchars
	return 0 if ($o > 126); # high chars
	return 0 if (index('&<>"',$c) > -1); # special chars
	return 1;
}

sub xml_escape
{
	my $in = shift;
	return '' unless defined $in;
	my $out = '';
	my $char;
	# PARAMETER: plain utf8 string with no special meaning
	# RETURNS: escaped string suitable for XML attrib or text
	# replace each character in the string with its escaped version unless it's
	# XML-safe ASCII
	my $count;
	for($count = 0;$count<length($in);$count++)
	{
		$char = substr($in,$count,1);
		$char = ($char eq "<")? '&lt;':
		        ($char eq ">")? '&gt;':
		        ($char eq '"')? '&quot;':
		        '&#'.ord($char).';'
		  unless is_xml_safe($char);
		$out .= $char;
	}
	return $out;
}

sub xml
{
	my $self = shift;
	return xml_escape($self->{text});
}

sub new
{
	my $class = shift;
	die "no class!" unless $class;
	my $self = { text => shift };
	$self->{parent} = shift;
	return bless $self, $class;
}

sub text
{
	my $self = shift;
	my $val = shift;
	return undef unless ref($self);
	$self->{text} = $val if defined $val;
	return $self->{text};
}

sub force_change_parent
{
	my $self = shift;
	my $new_parent = shift;
	$self->{parent} = $new_parent;
	return $self->{parent};
}

sub append
{
	my $self = shift;
	my $val = shift;
	return undef unless ref($self);
	$self->{text} .= $val if defined $val;
	return $self; # for multiple appending ;-)
}

1;
