package Pseudorandom_co_uk::Pseudosite::RedirectConstructor;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

use strict;
use warnings;
use subs qw();
use vars qw(@ISA);
use utf8;

require Pseudorandom_co_uk::Pseudosite::Constructor;
@ISA = qw(Pseudorandom_co_uk::Pseudosite::Constructor);

sub construct
{
	my $self = shift;
	my $node = shift;

	my $out_name = $node->attr('outfile');

	my $url = $node->attr('target');
	my $other = $self->processor->find_node($url);

	$url = ( $node->attr('server') eq $other->attr('server') ?
	         $other->attr('url-local')
	       : $other->attr('url') );

	if ( (! -f $out_name) or $self->processor->thorough )
	{
		print "redirect: $out_name\n" if $self->verbosity > 0;

		my $dir = $out_name;
		my @list;
		while( $dir =~ m!/[^/]+$! )
		{
			$dir =~ s!^(.*)/[^/]+$!$1!;
			unshift @list, $dir;
		}
		foreach $dir (@list)
		{
			unless (-d $dir or $dir eq '')
			{
				print "$dir doesn't exist, creating it...\n" if $self->verbosity > 0;
				mkdir $dir or die "Couldn't create $dir";
			}
		}
		local $\;
		local *OUTFILE;
		$\ = '';
		if(open OUTFILE, '>', $out_name)
		{
			print OUTFILE <<"END";
<html><head><title>redirection</title>
<meta http-equiv="refresh" content="0; URL=$url" />
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<script language="Javascript">
function pseudosite_redirect() { top.location.href = "$url" }
</script>
</head>
<body onLoad="pseudosite_redirect()"><p>redirecting to $url; if nothing happens or you
don't have Javascript, click <a href="$url">here</a></p></body></html>
END
			close OUTFILE;
		} else {
			die "Couldn't open $out_name for writing";
		}
	}
}
1;
