package Pseudorandom_co_uk::Pseudosite::PseudorandomExtension;

# Pseudosite copyright (c) 2002 Simon McVittie
# This module is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2,
# as published by the Free Software Foundation.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# Last change and base version. Please edit the last change when you change things.
# Please do not edit the base version unless you are the author.

# Base version (for patch purposes): v0.1 20020403 smcv@pseudorandom.co.uk
# Changelog since release (newest first):
# 20020403 smcv@pseudorandom.co.uk
#	Preparation for initial release.

# ====================================================================

# The pseudorandom-specific bits other people won't want, except as an example.

# WARNING: code in this file is non-standard. Many parts of this file rely on features
# of pseudorandom.co.uk which are not present in the examples (icons in a hard-coded
# location, etc.).

# Also, this script assumes the root node has an ID of "/".

# Anyway, this is how you plug in extensions. The initialization code (at the end) will
# be run with the global variable $_ set to a reference to the processor object.

# ============== EXTRA TAGS ==================

sub process_h0
{
	# pseudorandom-specific extra heading
	# h0 is defined to be an extra-big h1 for use on section-of-site headings
	my $cons = shift;
	my $tag = shift;
	my $retval = '';
	unless( $cons->state_pre or $cons->state_newline)
	{
		$retval .= "\n";
	}
	$retval .= '<h1 class="bighd"';
	my $attr;
	my $at;
	my $val;
	foreach $attr ($tag->attr_list)
	{
		my $at = $cons->xml_escape($attr);
		$val = $cons->xml_escape($tag->attr($attr));
		$retval .= qq( $at="$val") unless $at eq 'class';
	}
	$retval .= ">";
	$retval .= $cons->process_children_of($tag);
	$retval .= '</h1>';
	if( $cons->state_pre or $cons->state_newline)
	{
		$cons->state_newline(0);
	} else {
		$retval .= "\n";
		$cons->state_newline(1);
	}
	return $retval;
}

sub process_ll
{
	# "Literal link" left over from earlier versions.
	warn "<ll> is deprecated";
	my $self = shift;
	my $tag = shift;
	$self->process_l($tag);
}


sub process_il
{
	# Left over from earlier versions.
	warn "<il> is deprecated";
	my $self = shift;
	my $tag = shift;
	$tag->attr('href','|'.$tag->attr('file'));
	$self->process_l($tag);
}

sub process_el
{
	# Left over from earlier versions.
	warn "<el> is deprecated";
	my $self = shift;
	my $tag = shift;
	$self->process_l($tag);
}

sub process_fullGPL
{
	# The GNU GPL in all its rather long glory
	# In the real version, requires SSI and files in a hard-coded location
	my $self = shift;
	$self->state_newline(1);
	return "<p>(The GNU GPL would go here in the real version)</p>\n";
}
sub process_quickGPL
{
	# The statement saying "this is GPLed, no warranty"
	# Requires SSI and files in a hard-coded location
	my $self = shift;
	$self->state_newline(1);
	return "<p>(a quick statement of the GNU GPL would go here in the real version)</p>\n";
}

sub process_div
{
	# weird stuff to be done to <div/> so I can have OS-specific <div>s
	# with an appropriate icon
	# this won't work in the example site, since the icons are missing
	my $c = shift;
	my $tag = shift;
	my $retval = '';
	unless( $c->state_pre or $c->state_newline)
	{
		$retval .= "\n";
	}
	$retval .= '<div ';
	my $attr;
	my $val;
	my $at;
	foreach $attr ($tag->attr_list)
	{
		$at = $c->xml_escape($attr);
		$val = $c->xml_escape($tag->attr($attr));
		$retval .= " $attr=\"$val\"";
	}
	$retval .= ">";
	$c->state_newline(0);
	my $class = $tag->attr('class');
	if($class)
	{
		if($class eq 'linux')
		{
			$retval .= '<img src="'
			  .$c->processor->link_address(0,'ps/img/tux32x32.png')
			  .'" alt="[image: Linux logo]" width="32" height="32" class="f" title="Linux" />';
		} elsif($class eq 'mac') {
			$retval .= '<img src="'
			  .$c->processor->link_address(0,'ps/img/mac32x32.png')
			  .'" alt="[image: Apple logo]" width="32" height="32" class="f" title="MacOS" />';
		} elsif($class eq 'windows') {
			$retval .= '<img src="'
			  .$c->processor->link_address(0,'ps/img/win32x32.png')
			  .'" alt="[image: Windows logo]" width="32" height="32" class="f" title="Win32" />';
		}
	}
	$retval .= $c->process_children_of($tag);
	$retval .= '</div>';
	if( $c->state_pre or $c->state_newline)
	{
		$c->state_newline(0);
	} else {
		$retval .= "\n";
		$c->state_newline(1);
	}
	return $retval;
}

# ================ AUTOMAGIC SITE MAP

sub process_sitemap_node
{
	my $c = shift;
	my $node = shift;
	my $retval = '';
	my $title;
	if(ref $node and UNIVERSAL::isa($node,'Pseudorandom_co_uk::Pseudosite::PseudoTreeNode'))
	{
		if($node->fulltype eq '{http://www.pseudorandom.co.uk/xmlns/pseudosite}node' and $node->attr('indexed') and $node->attr('indexed') ne 'no')
		{
			if(!$c->processor->is_no($node->attr('linked')))
			{
				$retval .= '<li><a title="';
				$retval .= $c->xml_escape($node->attr('title'));
				$retval .= '" href="';
				$retval .= $c->xml_escape(
						( ( $c->processor->current_node->attr('server') and $node->attr('server')
						  and $c->processor->current_node->attr('server') eq $node->attr('server') ) ?
						    $node->attr('url-local')
						  : $node->attr('url') )
					);
				$retval .= '">';
			} else {
				$retval .= '<li><span title="';
				$retval .= $c->xml_escape($node->attr('title'));
				$retval .= '">';
			}
			$title = $node->attr('context-title');
			$title = $node->attr('title') unless $title;
			$title = $node->attr('short-title') unless $title;
			$retval .= $c->xml_escape($title);
			if($node->attr('linked') and $node->attr('linked') ne 'no')
			{
				$retval .= "</a>";
			} else {
				$retval .= "</span>";
			}
		}
		my @kids = ();
		my $output = '';
		@kids = @{$node->kids()};
		if(@kids)
		{
			while(@kids)
			{
				$output .= process_sitemap_node($c,shift @kids);
			}
			if($output)
			{
				$retval .= "<ul>\n$output</ul>";
			}
		}
		if($node->fulltype eq '{http://www.pseudorandom.co.uk/xmlns/pseudosite}node' and $node->attr('indexed') and $node->attr('indexed') ne 'no')
		{
			$retval .= "</li>\n";
		}
	}
	return $retval;
}

sub inside_content_after_sitemap
{
	my $c=shift;
	return "<ul>\n"
		. process_sitemap_node($c,$c->processor->find_node('/'))
		. "</ul>\n"
		.'</div>'."\n";
}


sub sitemap_tag
{
	my $c = shift;
	my $tag = shift;
	my $base = $tag->attr('root');
	my $node = $c->processor->find_node($base);
	my $retval;

	$retval = "<ul>\n"
		. process_sitemap_node($c,$node)
		. "</ul>\n";
	return $retval;
}

# =============== AUTOMAGIC "YOU ARE HERE"

sub you_are_here
{
	my $c = shift;
	my $curr = $c->processor->current_node;
	my $title = $curr->attr('short-title');
	$title = $curr->attr('title') unless $title;
	die "no title for ".$curr->attr('id') unless $title;
	my $retval = $c->xml_escape($title);
	my $temp;
	my $node;
	for($node = $curr->parent; $node and UNIVERSAL::isa($node,'Pseudorandom_co_uk::Pseudosite::PseudoTreeNode'); $node = $node->parent)
	{

		next unless ($node->ns eq 'http://www.pseudorandom.co.uk/xmlns/pseudosite');
		next unless ($node->attr('indexed') and $node->attr('indexed') ne 'no');

		$temp = $node->attr('short-title');
		$temp = $node->attr('context-title') unless $temp;
		$temp = $node->attr('title') unless $temp;
		$title = $node->attr('title');
		$title = $node->attr('context-title') unless $title;
		$title = $node->attr('short-title') unless $title;

		if(! ($c->processor->is_no($node->attr('linked'))) )
		{
			$temp = "<a href=\""
				. $c->xml_escape(
				( $c->processor->current_node->attr('server')
				and $node->attr('server') and
				 ( $c->processor->current_node->attr('server')
				eq $node->attr('server') ) ) ?
					$node->attr('url-local')
					: $node->attr('url')
				)
				. '" title="'
				. $c->xml_escape($title)
				. "\">" . $temp . "</a>";
		} else {
			$temp = '<span title="'
				. $c->xml_escape($title)
				. "\">" . $temp . "</span>";
		}
		$retval = $temp . ' :: ' . $retval;
	}
	return "you are here: $retval\n";
}

# ================================== BEFORE THE CONTENT

sub webauth_navbar
{
	my $self = shift;
	return main_navbar($self,@_) . "(The Web Design navbar goes here)\n";
}

sub main_navbar
{
	my $c = shift;
	my $retval = "<div class=\"m\"  id=\"tbannerdiv\">"
	       . "(The logo goes here)\n"
	       . "(The navbar goes here)\n"
	       . "<div class=\"youarehere\">\n"
	       . you_are_here($c)
	       . "</div></div>\n"
	       . "<hr class=\"old\" />\n";
	return $retval;
}


sub paradox_navbars
{
	my $c = shift;
	return "<div class=\"m\" id=\"minibanner\">"
	       . "(The navbar goes here)\n"
	       . "</div><div class=\"n\">\n"
	       . "(The Paradox Productions logo goes here)\n"
	       . "(The Paradox Productions navbar goes here)\n"
	       . "<div class=\"youarehere\">\n"
	       . you_are_here($c)
	       . "</div></div>\n"
	       . "<hr class=\"old\" />\n";
}

# ===================== THE BEGINNING OF THE CONTENT

sub begin_content_common
{
	return '<div class="t">'."\n";
}

sub begin_paradox_archive
{
	my $c = shift;
	my $ret = begin_content_common($c);
	$ret .= "<p class=\"n\"><i><b>This page has been archived. Content in "
	       . "this section is likely to be out of date or never updated.</b></i></p>\n"
	  if $c->processor->current_node->attr('id') ne 'paradox/archive';
	return $ret;
}

# ============================ END OF THE CONTENT

sub end_content
{
	return '</div>'."\n";
}

# ============================ AFTER THE CONTENT

sub after_content
{
	my $c = shift;
	return "<hr class=\"old\" />\n"
	       . "<div class=\"m\">\n"
	       . you_are_here($c)
	       . "</div>\n"
	       ;
}

# ===================== INSTALLATION CODE

sub cssmaker
{
	# these closure thingies are useful ;-)
	my $str = shift;
	return sub {
		my $self = shift;
		return "<style type=\"text/css\">\n".'@import url('
		.$self->processor->link_address(0,$str,1)
		.");\n</style>\n";
	};
}

my $proc = $_;
my $const;
my $class;

foreach $class ($proc->list_classobjs)
{
	$const = $class->{constructor};
	if($const->isa("Pseudorandom_co_uk::Pseudosite::PseudoHTMLSimple"))
	{
		# if this script gets modified, most stuff will need changing
		$const->add_everything_dependency('example/pseudorandom_extension.pl');

		# leftovers from previous versions; just ignore them and process their
		# content
		$const->pseudohtml_container('synopsis',1);
		$const->pseudohtml_container('readme', 1);
		$const->pseudohtml_container('no-readme', 1);
		$const->pseudohtml_container('ssi',1);
		$const->pseudohtml_ignored('no-online',1);
		# Additional tags
		$const->pseudohtml_handlers('div',\&process_div);
		$const->pseudohtml_handlers('h0', \&process_h0);
		$const->pseudohtml_handlers('quickGPL',\&process_quickGPL);
		$const->pseudohtml_handlers('fullGPL',\&process_fullGPL);
		$const->pseudohtml_handlers('sitemap',\&sitemap_tag);
		$const->pseudohtml_handlers('ll', \&process_ll);
		$const->pseudohtml_handlers('il', \&process_il);
		$const->pseudohtml_handlers('el', \&process_el);
		$const->pseudohtml_handlers('ent', \&Pseudorandom_co_uk::Pseudosite::PseudoHTMLSimple::process_e);
		# Section handlers (default)
		$const->section_handlers('before_content', \&main_navbar);
		$const->section_handlers('begin_content', \&begin_content_common);
		$const->section_handlers('end_content', \&end_content);
		$const->section_handlers('after_content', \&after_content);
		$const->section_handlers('css_block', cssmaker('ps/css/base.css'));
		# Section handlers (private)
		$const->style_section_handlers('paradox','before_content', \&paradox_navbars);
		$const->style_section_handlers('paradox_archive','before_content', \&paradox_navbars);
		$const->style_section_handlers('paradox_archive','begin_content', \&begin_paradox_archive);
		my $css = cssmaker('ps/css/paradox.css');
		$const->style_section_handlers('paradox','css_block', $css);
		$const->style_section_handlers('paradox_archive','css_block', $css);

		$const->style_section_handlers('webauth','css_block', cssmaker('ps/css/webauth.css'));
		$const->style_section_handlers('webauth','before_content', \&webauth_navbar);
		$const->style_section_handlers('maths','css_block', cssmaker('ps/css/maths.css'));
		$const->style_section_handlers('sitemap','end_content', \&inside_content_after_sitemap);
	}
}


1;
